@@ -1,4 +1,5 @@
 use std::path::PathBuf;
+use std::collections::HashSet;
 
 use serde::{Serialize,Deserialize};
 
@@ -11,12 +12,38 @@ pub struct Logfile {
     pub generations: Option<u64>,
 }
 
+#[derive(PartialEq,Eq,Debug,Serialize,Deserialize,Hash)]
+#[serde(rename_all = "lowercase")]
+pub enum ArrayOrString { Array, String }
+
+impl Default for ArrayOrString {
+    fn default() -> Self { ArrayOrString::Array }
+}
+
+#[derive(Default,Debug,Serialize,Deserialize)]
+pub struct Transform {
+    #[serde(rename="execve-argv")] #[serde(default)]
+    pub execve_argv: HashSet<ArrayOrString>
+}
+
+#[derive(Default,Debug,Serialize,Deserialize)]
+pub struct Enrich {}
+
+#[derive(Default,Debug,Serialize,Deserialize)]
+pub struct Filter {}
+
 #[derive(Debug,Serialize,Deserialize)]
 pub struct Config {
     pub user: Option<String>,
     pub directory: Option<PathBuf>,
     pub auditlog: Logfile,
     pub debuglog: Option<Logfile>,
+    #[serde(default)]
+    pub transform: Transform,
+    #[serde(default)]
+    pub enrich: Enrich,
+    #[serde(default)]
+    pub filter: Filter,
 }
 
 impl Default for Config {
@@ -31,6 +58,9 @@ impl Default for Config {
                 generations: None,
             },
             debuglog: None,
+            transform: Transform::default(),
+            enrich: Enrich::default(),
+            filter: Filter::default(),
         }
     }
 }
